/*
 *  $Id: pvrusb2-hdw-internal.h,v 1.5 2005/05/06 04:22:46 isely Exp $
 *
 *  Copyright (C) 2005 Mike Isely <isely@pobox.com>
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 */
#ifndef __PVRUSB2_HDW_INTERNAL_H
#define __PVRUSB2_HDW_INTERNAL_H

#include <linux/videodev2.h>
#include <linux/i2c.h>
#include <media/tveeprom.h>
#include <asm/semaphore.h>
#include "pvrusb2-hdw.h"

#define PVR2_CTL_BUFFSIZE        0x0100

#define FREQTABLE_SIZE 500

struct pvr2_ctl_state {
    int value;
    int dirty;
};

/* This structure contains all state data directly needed to
   manipulate the hardware (as opposed to complying with a kernel
   interface) */
struct pvr2_hdw {
    /* Underlying USB device handle */
    struct usb_device *usb_dev;
    struct usb_interface *usb_intf;

    /* Mutex for all hardware state control */
    struct semaphore big_lock;

    char name[32];

    /* I2C stuff */
    struct i2c_adapter i2c_adap;
    struct i2c_algorithm i2c_algo;
    int i2c_linked;
    struct i2c_client *i2c_tuner_client;
    struct i2c_client *i2c_audio_client;
    struct i2c_client *i2c_video_client;

    /* Frequency table */
    unsigned int freqTable[FREQTABLE_SIZE];

    /* our control pool */
    struct semaphore ctl_sem;
    unsigned char cmd_buffer[PVR2_CTL_BUFFSIZE];  /* used under ctl_sem lock */

    int flag_ok;            // device in known good state
    int flag_disconnected;  // flag_ok == 0 due to disconnect
    int flag_streaming;     // currently streaming
    int flag_init_ok;       // true if structure is fully initialized

    /* Tuner / frequency control stuff */
    struct tveeprom tvdata;

    enum pvr2_config config;

    /* Information about what audio signal we're hearing */
    int flag_stereo;
    int flag_bilingual;

    /* Every last bit of controllable state */
    struct pvr2_ctl_state controls[PVR2_CID_COUNT];
};

int pvr2_hdw_set_ctl_value_internal(struct pvr2_hdw *hdw,
                                    unsigned int ctl_id,int value);
int pvr2_hdw_commit_ctl_internal(struct pvr2_hdw *hdw);


#endif /* __PVRUSB2_HDW_INTERNAL_H */
/*
  Stuff for Emacs to see, in order to encourage consistent editing style:
  *** Local Variables: ***
  *** mode: c ***
  *** fill-column: 75 ***
  *** c-basic-offset: 4 ***
  *** indent-tabs-mode: nil ***
  *** End: ***

  Similarly for vim to see:
  vim:expandtab:textwidth=75

*/
