/*
 *  $Id: pvrusb2-hdw.h,v 1.5 2005/05/06 04:22:46 isely Exp $
 *
 *  Copyright (C) 2005 Mike Isely <isely@pobox.com>
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 */
#ifndef __PVRUSB2_HDW_H
#define __PVRUSB2_HDW_H

#include <linux/usb.h>

/* Definition of state variables that we can inspect & change.  Numbers are
   assigned from zero counting up with no gaps. */
#define PVR2_CID_BRIGHTNESS 0
#define PVR2_CID_CONTRAST 1
#define PVR2_CID_SATURATION 2
#define PVR2_CID_HUE 3
#define PVR2_CID_VOLUME 4
#define PVR2_CID_BALANCE 5
#define PVR2_CID_BASS 6
#define PVR2_CID_TREBLE 7
#define PVR2_CID_MUTE 8
#define PVR2_CID_SRATE 9
#define PVR2_CID_AUDIOBITRATE 10
#define PVR2_CID_AUDIOCRC 11
#define PVR2_CID_AUDIOEMPHASIS 12
#define PVR2_CID_VBR 13
#define PVR2_CID_AVERAGEVIDEOBITRATE 14
#define PVR2_CID_PEAKVIDEOBITRATE 15
#define PVR2_CID_VIDEOSTANDARD 16
#define PVR2_CID_INPUT 17
#define PVR2_CID_AUDIOMODE 18
#define PVR2_CID_FREQUENCY 19 // Units of Hz
#define PVR2_CID_HRES 20
#define PVR2_CID_VRES 21
#define PVR2_CID_INTERLACE 22
#define PVR2_CID_AUDIOLAYER 23
#define PVR2_CID_CHANNEL 24
#define PVR2_CID_CHANPROG_ID 25
#define PVR2_CID_CHANPROG_FREQ 26

/* Number of state variables */
#define PVR2_CID_COUNT 27


/* Legal values for the SRATE state variable */
#define PVR2_CVAL_SRATE_48 0
#define PVR2_CVAL_SRATE_44_1 1
#define PVR2_CVAL_SRATE_MIN PVR2_CVAL_SRATE_48
#define PVR2_CVAL_SRATE_MAX PVR2_CVAL_SRATE_44_1

/* Legal values for the AUDIOBITRATE state variable */
#define PVR2_CVAL_AUDIOBITRATE_384 0
#define PVR2_CVAL_AUDIOBITRATE_320 1
#define PVR2_CVAL_AUDIOBITRATE_256 2
#define PVR2_CVAL_AUDIOBITRATE_224 3
#define PVR2_CVAL_AUDIOBITRATE_192 4
#define PVR2_CVAL_AUDIOBITRATE_160 5
#define PVR2_CVAL_AUDIOBITRATE_128 6
#define PVR2_CVAL_AUDIOBITRATE_112 7
#define PVR2_CVAL_AUDIOBITRATE_96 8
#define PVR2_CVAL_AUDIOBITRATE_80 9
#define PVR2_CVAL_AUDIOBITRATE_64 10
#define PVR2_CVAL_AUDIOBITRATE_56 11
#define PVR2_CVAL_AUDIOBITRATE_48 12
#define PVR2_CVAL_AUDIOBITRATE_32 13
#define PVR2_CVAL_AUDIOBITRATE_VBR 14
#define PVR2_CVAL_AUDIOBITRATE_MIN PVR2_CVAL_AUDIOBITRATE_384
#define PVR2_CVAL_AUDIOBITRATE_MAX PVR2_CVAL_AUDIOBITRATE_VBR

/* Legal values for the AUDIOEMPHASIS state variable */
#define PVR2_CVAL_AUDIOEMPHASIS_NONE 0
#define PVR2_CVAL_AUDIOEMPHASIS_50_15 1
#define PVR2_CVAL_AUDIOEMPHASIS_CCITT 2
#define PVR2_CVAL_AUDIOEMPHASIS_MIN PVR2_CVAL_AUDIOEMPHASIS_NONE
#define PVR2_CVAL_AUDIOEMPHASIS_MAX PVR2_CVAL_AUDIOEMPHASIS_CCITT

/* Legal values for the VIDEOSTANDARD state variable */
#define PVR2_CVAL_VIDEOSTANDARD_NTSC_M 0
#define PVR2_CVAL_VIDEOSTANDARD_SECAM_L 1
#define PVR2_CVAL_VIDEOSTANDARD_PAL_BG 2
#define PVR2_CVAL_VIDEOSTANDARD_PAL_I 3
#define PVR2_CVAL_VIDEOSTANDARD_PAL_DK 4
#define PVR2_CVAL_VIDEOSTANDARD_MIN PVR2_CVAL_VIDEOSTANDARD_NTSC_M
#define PVR2_CVAL_VIDEOSTANDARD_MAX PVR2_CVAL_VIDEOSTANDARD_PAL_DK

/* Legal values for the INPUT state variable */
#define PVR2_CVAL_INPUT_TV 0
#define PVR2_CVAL_INPUT_SVIDEO 1
#define PVR2_CVAL_INPUT_COMPOSITE 2
#define PVR2_CVAL_INPUT_RADIO 3
#define PVR2_CVAL_INPUT_MIN PVR2_CVAL_INPUT_TV
#define PVR2_CVAL_INPUT_MAX PVR2_CVAL_INPUT_RADIO

/* Legal values for the AUDIOMODE state variable */
#define PVR2_CVAL_AUDIOMODE_MONO 0
#define PVR2_CVAL_AUDIOMODE_STEREO 1
#define PVR2_CVAL_AUDIOMODE_SAP 2
#define PVR2_CVAL_AUDIOMODE_LANG1 3
#define PVR2_CVAL_AUDIOMODE_LANG2 4
#define PVR2_CVAL_AUDIOMODE_MIN PVR2_CVAL_AUDIOMODE_MONO
#define PVR2_CVAL_AUDIOMODE_MAX PVR2_CVAL_AUDIOMODE_LANG2

/* Values that pvr2_hdw_get_signal_status() returns */
#define PVR2_SIGNAL_OK     0x0001
#define PVR2_SIGNAL_STEREO 0x0002
#define PVR2_SIGNAL_SAP    0x0004

enum pvr2_config {
    pvr2_config_empty,
    pvr2_config_mpeg,
    pvr2_config_vbi,
    pvr2_config_radio,
};

struct pvr2_hdw;

/* Create and return a structure for interacting with the underlying
   hardware */
struct pvr2_hdw *pvr2_hdw_create(struct usb_interface *intf);

/* Destroy hardware interaction structure */
void pvr2_hdw_destroy(struct pvr2_hdw *);

/* Set up the structure and attempt to put the device into a usable state.
   This can be a time-consuming operation, which is why it is not done
   internally as part of the create() step.  Return value is exactly the
   same as pvr2_hdw_init_ok(). */
int pvr2_hdw_setup(struct pvr2_hdw *);

/* Initialization succeeded */
int pvr2_hdw_init_ok(struct pvr2_hdw *);

/* Return true if in the ready (normal) state */
int pvr2_hdw_dev_ok(struct pvr2_hdw *);

/* Get pointer to underlying USB device */
struct usb_device *pvr2_hdw_get_dev(struct pvr2_hdw *);

/* Retrieve serial number of device */
unsigned long pvr2_hdw_get_sn(struct pvr2_hdw *);

/* Called when hardware has been unplugged */
void pvr2_hdw_disconnect(struct pvr2_hdw *);

/* Retrieve current value for a given control */
int pvr2_hdw_get_ctl_value(struct pvr2_hdw *,unsigned int ctl_id);

/* Retrieve legal minimum value for a given control */
int pvr2_hdw_get_ctl_min_value(struct pvr2_hdw *,unsigned int ctl_id);

/* Retrieve legal maximum value for a given control */
int pvr2_hdw_get_ctl_max_value(struct pvr2_hdw *,unsigned int ctl_id);

/* Set current value for given control - this is just stored; the hardware
   isn't updated until the commit function is called. */
int pvr2_hdw_set_ctl_value(struct pvr2_hdw *,unsigned int ctl_id,int value);

/* Retrieve string name for given control */
const char *pvr2_hdw_get_ctl_name(struct pvr2_hdw *,unsigned int ctl_id);

/* Retrieve string name for a given control value (returns a null pointer
   for any invalid combinations). */
const char *pvr2_hdw_get_ctl_value_name(struct pvr2_hdw *,
                                        unsigned int ctl_id,int value);

/* Commit all control changes made up to this point */
int pvr2_hdw_commit_ctl(struct pvr2_hdw *);

/* Find out how many controls there are.  Legal ids are numbered from 0
   through this value - 1. */
unsigned int pvr2_hdw_get_ctl_count(struct pvr2_hdw *);

/* Return PVR2_SIGNAL_XXXX bit mask indicating signal status */
unsigned int pvr2_hdw_get_signal_status(struct pvr2_hdw *);

/* Turn streaming on/off */
int pvr2_hdw_set_streaming(struct pvr2_hdw *,int);

/* Configure the type of stream to generate */
int pvr2_hdw_set_stream_type(struct pvr2_hdw *, enum pvr2_config);

#ifdef notdef
/* Get handle to video output stream */
struct pvr2_stream *pvr2_hdw_get_video_stream(struct pvr2_hdw *);
#endif


#endif /* __PVRUSB2_HDW_H */
/*
  Stuff for Emacs to see, in order to encourage consistent editing style:
  *** Local Variables: ***
  *** mode: c ***
  *** fill-column: 75 ***
  *** c-basic-offset: 4 ***
  *** indent-tabs-mode: nil ***
  *** End: ***

  Similarly for vim to see:
  vim:expandtab:textwidth=75

*/
