/*
 *
 *  Copyright (C) 2005 Mike Isely <isely@pobox.com>
 *  Copyright (C) 2004 Aurelien Alleaume <slts@free.fr>
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 */

#include "pvrusb2-hdw-low.h"
#include "pvrusb2-tuner.h"
#include "pvrusb2-hdw-internal.h"
#include "pvrusb2-i2c.h"
#include "pvrusb2-debug.h"
#include <linux/videodev.h>
#include <media/tuner.h>

#define PVR_STD_I2C_ADDR    0x43

static u32 pvr_tbl_standard[] = {
    [PVR2_CVAL_VIDEOSTANDARD_PAL_BG]  = 0x00d47009, /* PVR_STD_PAL_BG */
    [PVR2_CVAL_VIDEOSTANDARD_PAL_I]   = 0x00d4700a, /* PVR_STD_PAL_I */
    [PVR2_CVAL_VIDEOSTANDARD_PAL_DK]  = 0x00d4700b, /* PVR_STD_PAL_DK */
    [PVR2_CVAL_VIDEOSTANDARD_SECAM_L] = 0x00c4100b, /* PVR_STD_SECAM */
    [PVR2_CVAL_VIDEOSTANDARD_NTSC_M]  = 0x00163044  /* PVR_STD_NTSC */
};

#define trace_tuner(...) pvr2_trace(PVR2_TRACE_TUNER,__VA_ARGS__)

int pvr2_tuner_set_freq(struct pvr2_hdw *hdw)
{
    int stat;
    struct v4l2_frequency freq;
    freq.frequency = hdw->controls[PVR2_CID_FREQUENCY].value / 62500;
    freq.tuner = 0;
    freq.type = V4L2_TUNER_ANALOG_TV;
    trace_tuner("pvr2_tuner_set_freq(%d)",freq.frequency);
    stat = pvr2_i2c_tuner_cmd(hdw,VIDIOC_S_FREQUENCY,&freq);
    if (stat < 0) return stat;
    /* This kicks the audio controller... */
    stat = pvr2_i2c_msp3400_cmd(hdw,VIDIOCSFREQ,0);
    if (stat < 0) return stat;
    return 0;
}

int pvr2_tuner_set_type(struct pvr2_hdw *hdw)
{
    trace_tuner("pvr2_tuner_set_type(%d)",hdw->tvdata.tuner_type);
    if (((int)(hdw->tvdata.tuner_type)) < 0) {
	trace_tuner("tuner type not set yet");
	return 0;
    }
    return pvr2_i2c_tuner_cmd(hdw,TUNER_SET_TYPE,&hdw->tvdata.tuner_type);
}

int pvr2_tuner_set_standard(struct pvr2_hdw *hdw)
{
    struct i2c_msg msg[1];
    u8 buff [4];

    trace_tuner("pvr2_tuner_set_standard(%d)",
		hdw->controls[PVR2_CID_VIDEOSTANDARD].value);

    /* FIXME: Might be tuner dependant ? Need more hardware info... */
    PVR2_DECOMPOSE_BE(
	buff, 0,pvr_tbl_standard[hdw->controls[PVR2_CID_VIDEOSTANDARD].value]);

    msg[0].addr = PVR_STD_I2C_ADDR;
    msg[0].flags = 0;
    msg[0].len = sizeof(buff);
    msg[0].buf = buff;

    i2c_transfer(&hdw->i2c_adap,msg,sizeof(msg)/sizeof(msg[0]));

    return 0;
}
