/*
 *
 *  Copyright (C) 2005 Mike Isely <isely@pobox.com>
 *  Copyright (C) 2004 Aurelien Alleaume <slts@free.fr>
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 */
#ifndef __PVRUSB2_V4L2_INTERNAL_H
#define __PVRUSB2_V4L2_INTERNAL_H

#include "pvrusb2-v4l2.h"
#include <linux/videodev.h>


#define PVR_WIDTH_DVD      720
#define PVR_WIDTH_SVCD     480
#define PVR_WIDTH_VCD      352

#define PVR_HEIGHT_PAL      480
#define PVR_HEIGHT_NTSC     576

/* The next ones are indexes in v4l2 structs */

#define PVR_V4L2_NSTD   5

#define PVR_STD_PAL_BG  0
#define PVR_STD_PAL_I   1
#define PVR_STD_PAL_DK  2
#define PVR_STD_SECAM   3
#define PVR_STD_NTSC    4

#define PVR_V4L2_NINPUT    3
#define PVR_V4L2_AUDIO_NINPUT    2

#define PVR_INPUT_TV        0
#define PVR_INPUT_COMPOSITE 1
#define PVR_INPUT_SVIDEO    2
#define PVR_INPUT_RADIO     3

#define PVR_SRATE_48 0
#define PVR_SRATE_44 1

#define PVR_FORMAT_PIX  0
#define PVR_FORMAT_VBI  1

#define PVR_NSTREAM 4

/* Buffer management constants */
#define PVR_VID_BUFFSIZE        PAGE_ALIGN(0x4000)
#define PVR_NBUFF               32       /* Max number of video buffers */

struct pvr2_v4l2_fh;
struct pvr2_v4l2;

enum pvr_memory_state {
    _freed,
    _allocated,
    _mapped,    /* at least one buffer is mapped */
};

enum pvr_io_method {
    _none,
    _read,
    _mmap,
    _uptr
};

struct pvr_mapping {
	struct pvr2_v4l2_stream *stream;
	int count;
};

struct pvr_buffer {
    struct pvr2_v4l2_stream *stream;
    void *buff;     /* Allocated buffer. Page aligned one is in purb */
    u8 *data;     /* First unread data byte */
    size_t size;    /* Size of unread data (for read-io) */
    struct urb *purb;
    short submitted;
    struct pvr_mapping *map;    /* NULL if not mapped */
    int index;
};

struct pvr_uptr {
    unsigned long ptr;
    unsigned int length;
};

struct pvr2_v4l2_stream {
    struct pvr2_v4l2 *v4lp;
    struct video_device *vdev;
    int v4l_type;

    struct semaphore stream_sem;
    int user;

    unsigned long last_buffer_timestamp;
    enum pvr_memory_state mem_state;
    enum pvr_io_method io_method;
    struct pvr2_v4l2_fh *io_owner;
    enum pvr2_config config;

    int endpoint;   /* usb endpoint to read */
    atomic_t submitted_urbs;
    wait_queue_head_t urbs_collected;
    wait_queue_head_t wait_data;

    struct pvr_buffer buffer[PVR_NBUFF];        /* allocated buffers */
    int buffer_count;   /* how many buffer do we want ? */
    int map_count;      /* number of mapped buffers */
    struct pvr_buffer *rx_buffer[PVR_NBUFF];    /* ring of recv buffers */
    spinlock_t ring_lock;   /* we don't want simultaneous add to the ring. */
    unsigned int rx1;
    /* might want to use atomic for smp ?*/
    unsigned int volatile rx2;
    /* ring for user pointers (acces under sem lock)*/
    struct pvr_uptr userptr[PVR_NBUFF];
    unsigned int uptr1;
    unsigned int uptr2;
    int align;
};

struct pvr2_v4l2_fh {
    struct pvr2_v4l2_stream *stream;
    enum v4l2_priority prio;
    struct file *file;
    struct pvr2_v4l2 *vhead;
    struct pvr2_v4l2_fh *vnext;
    struct pvr2_v4l2_fh *vprev;
};

struct pvr2_v4l2 {
    struct pvr2_channel channel;
    struct pvr2_v4l2_fh *vfirst;
    struct pvr2_v4l2_fh *vlast;

    struct v4l2_prio_state prio;

    /* streams */
    struct pvr2_v4l2_stream* stream[PVR_NSTREAM];
    int stream_count;
};

int pvr2_v4l2_ioctl(struct inode *inode, struct file *file,
                    unsigned int cmd, unsigned long arg);
int pvr2_v4l2_release(struct inode *inode, struct file *file);
int pvr2_v4l2_open(struct inode *inode, struct file *file);

#endif /* __PVRUSB2_V4L2_INTERNAL_H */
/*
  Stuff for Emacs to see, in order to encourage consistent editing style:
  *** Local Variables: ***
  *** mode: c ***
  *** fill-column: 75 ***
  *** c-basic-offset: 4 ***
  *** indent-tabs-mode: nil ***
  *** End: ***

  Similarly for vim to see:
  vim:expandtab:textwidth=75

*/
